/*
 HardwareSerial.h - Hardware serial library for Wiring
 Copyright (c) 2006 Nicholas Zambetti.  All right reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 Modified 28 September 2010 by Mark Sproul
 Modified 14 August 2012 by Alarus
 Modified 3 December 2013 by Matthijs Kooijman
 Modified 18 December 2014 by Ivan Grokhotkov (esp8266 platform support)
 Modified 31 March 2015 by Markus Sattler (rewrite the code for UART0 + UART1 support in ESP8266)
 */

#ifndef HardwareSerial_h
#define HardwareSerial_h

#include <inttypes.h>

#include "Stream.h"

#define SERIAL_TX_BUFFER_SIZE 256
#define SERIAL_RX_BUFFER_SIZE 256

// // Define config for Serial.begin(baud, config);
// #define SERIAL_5N1 0x00
// #define SERIAL_6N1 0x02
// #define SERIAL_7N1 0x04
// #define SERIAL_8N1 0x06
// #define SERIAL_5N2 0x08
// #define SERIAL_6N2 0x0A
// #define SERIAL_7N2 0x0C
// #define SERIAL_8N2 0x0E
// #define SERIAL_5E1 0x20
// #define SERIAL_6E1 0x22
// #define SERIAL_7E1 0x24
// #define SERIAL_8E1 0x26
// #define SERIAL_5E2 0x28
// #define SERIAL_6E2 0x2A
// #define SERIAL_7E2 0x2C
// #define SERIAL_8E2 0x2E
// #define SERIAL_5O1 0x30
// #define SERIAL_6O1 0x32
// #define SERIAL_7O1 0x34
// #define SERIAL_8O1 0x36
// #define SERIAL_5O2 0x38
// #define SERIAL_6O2 0x3A
// #define SERIAL_7O2 0x3C
// #define SERIAL_8O2 0x3E

class cbuf;

typedef enum {
    UART0 = 0, UART1 = 1, UART_NO = 0xFF
} UARTnr_t;

typedef struct {
        UARTnr_t uart_nr;
        int baud_rate;
        bool rxEnabled;
        bool txEnabled;
        uint8_t rxPin;
        uint8_t txPin;
} uart_t;

class HardwareSerial: public Stream {
    public:
        HardwareSerial(UARTnr_t uart_nr);

        void begin(unsigned long baud) {
            begin(baud, 0);
        }
        void begin(unsigned long, uint8_t);
        void end();
        void swap();  //use GPIO13 and GPIO15 as RX and TX
        int available(void) override;
        int peek(void) override;
        int read(void) override;
        int availableForWrite(void);
        void flush(void) override;
        size_t write(uint8_t) override;
        inline size_t write(unsigned long n) {
            return write((uint8_t) n);
        }
        inline size_t write(long n) {
            return write((uint8_t) n);
        }
        inline size_t write(unsigned int n) {
            return write((uint8_t) n);
        }
        inline size_t write(int n) {
            return write((uint8_t) n);
        }
        using Print::write; // pull in write(str) and write(buf, size) from Print
        operator bool() const;

        void setDebugOutput(bool);
        bool isTxEnabled(void);
        bool isRxEnabled(void);

    protected:
        friend void uart_interrupt_handler(uart_t* uart);
        void _rx_complete_irq(char c);
        void _tx_empty_irq(void);

    protected:
        UARTnr_t _uart_nr;
        uart_t* _uart;
        cbuf* _tx_buffer;
        cbuf* _rx_buffer;
        bool _written;
};

extern HardwareSerial Serial;
extern HardwareSerial Serial1;

#endif
